"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Unicorn(Monster):
    """
    Charge.
      If the unicorn moves at least 20 ft. straight toward a target and then
      hits it with a horn attack on the same turn, the target takes an extra
      9 (2d8) piercing damage. If the target is a creature, it must succeed
      on a DC 15 Strength saving throw or be knocked prone.
    Innate Spellcasting.
      The unicorn's innate spellcasting ability is Charisma (spell save DC
      14). The unicorn can innately cast the following spells, requiring no
      components:
      
  
      
      At will: detect evil and good, druidcraft, pass without trace
      
      1/day each: calm emotions, dispel evil and good, entangle
    Magic Resistance.
      The unicorn has advantage on saving throws against spells and other
      magical effects.
    Magic Weapons.
      The unicorn's weapon attacks are magical.
    Multiattack.
      The unicorn makes two attacks: one with its hooves and one with its
      horn.
    Hooves.
      Melee Weapon Attack: +7 to hit, reach 5 ft ., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    Horn.
      Melee Weapon Attack: +7 to hit, reach 5 ft ., one target. Hit: 8 (1d8
      + 4) piercing damage.
    Healing Touch.
      The unicorn touches another creature with its horn. The target
      magically regains 11 (2d8 + 2) hit points. In addition, the touch
      removes all diseases and neutralizes all poisons afflicting the
      target.
    Teleport.
      The unicorn magically teleports itself and up to three willing
      creatures it can see within 5 ft. of it, along with any equipment they
      are wearing or carrying, to a location the unicorn is familiar with,
      up to 1 mile away.
    """
    name = "Unicorn"
    description = "Large celestial, lawful good"
    challenge_rating = 5
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 13"
    languages = "Celestial, Elvish, Sylvan, telepathy 60 ft."
    strength = Ability(18)
    dexterity = Ability(14)
    constitution = Ability(15)
    intelligence = Ability(11)
    wisdom = Ability(17)
    charisma = Ability(16)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 67
    hit_dice = "9d10"
    spells = []
