from dungeonsheets.content_registry import default_content_registry

from dungeonsheets.monsters.monsters import *
from dungeonsheets.monsters.monsters_a import *
from dungeonsheets.monsters.monsters_b import *
from dungeonsheets.monsters.monsters_c import *
from dungeonsheets.monsters.monsters_d import *
from dungeonsheets.monsters.monsters_e import *
from dungeonsheets.monsters.monsters_f import *
from dungeonsheets.monsters.monsters_g import *
from dungeonsheets.monsters.monsters_h import *
from dungeonsheets.monsters.monsters_i import *
from dungeonsheets.monsters.monsters_j import *
from dungeonsheets.monsters.monsters_k import *
from dungeonsheets.monsters.monsters_l import *
from dungeonsheets.monsters.monsters_m import *
from dungeonsheets.monsters.monsters_n import *
from dungeonsheets.monsters.monsters_o import *
from dungeonsheets.monsters.monsters_p import *
from dungeonsheets.monsters.monsters_q import *
from dungeonsheets.monsters.monsters_r import *
from dungeonsheets.monsters.monsters_s import *
from dungeonsheets.monsters.monsters_t import *
from dungeonsheets.monsters.monsters_u import *
from dungeonsheets.monsters.monsters_v import *
from dungeonsheets.monsters.monsters_w import *
from dungeonsheets.monsters.monsters_x import *
from dungeonsheets.monsters.monsters_y import *
from dungeonsheets.monsters.monsters_z import *


default_content_registry.add_module(__name__)
