"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Jackal(Monster):
    """
    Keen Hearing and Smell.
      The jackal has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The jackal has advantage on an attack roll against a creature if at
      least one of the jackal's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +1 to hit, reach 5 ft., one target. Hit: 1 (1d4 -
      1) piercing damage.
    """
    name = "Jackal"
    description = "Small beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(8)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 3
    hit_dice = "1d6"
    spells = []
